<?php

namespace Drupal\batch_services;

use Drupal\Core\Messenger\MessengerInterface;

/**
 * A service for the Batch API.
 */
class BatchWorkerService implements BatchWorkerServiceInterface {
  /**
   * The number of items per result page.
   */
  const ITEMS_PER_PAGE = 100;

  /**
   * A Drupal Messenger object.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The batch context.
   *
   * @var array
   *   Batch context.
   */
  protected $context;

  /**
   * Contructs a BatchWorkerService object.
   */
  public function __construct(MessengerInterface $messenger) {
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public function getTitle() {
    return t('Processing items...');
  }

  /**
   * {@inheritdoc}
   */
  public function getErrorMessage() {
    return t('An error was encountered');
  }

  /**
   * Calculates whether the batch has finished processing.
   */
  public function calculate() {
    $progress = $this->getProgress();
    $total = $this->getTotal();
    if ($progress < $total) {
      $this->setPercentFinished($progress / $total * 100);
    }
    else {
      $this->setFinished();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function setContext(array &$context) {
    if (!isset($context['sandbox']['current_id'])) {
      $context['sandbox']['current_id'] = 0;
    }
    if (!isset($context['sandbox']['progress'])) {
      $context['sandbox']['progress'] = 0;
    }
    if (!isset($context['results']['total'])) {
      $context['results']['total'] = 0;
    }
    if (!isset($context['results']['created'])) {
      $context['results']['created'] = 0;
    }
    if (!isset($context['results']['updated'])) {
      $context['results']['updated'] = 0;
    }
    if (!isset($context['results']['skipped'])) {
      $context['results']['skipped'] = 0;
    }
    $this->context = &$context;
  }

  /**
   * Sets the batch progress message.
   */
  public function setProgressMessage($message = 'Processing item @current of @total...') {
    $total = $this->getTotal();
    if ($total) {
      $this->context['message'] = t($message, [
        '@current' => $this->getProgress(),
        '@total' => $total,
      ]);
    }
  }

  /**
   * Marks the batch process as being completely finished.
   */
  public function setFinished() {
    $this->setProgress($this->getTotal());
    $this->setPercentFinished(100);
  }

  /**
   * Mark the percent of the batch proccess complete.
   */
  public function setPercentFinished($percentage) {
    if ($percentage < 0 || $percentage > 100) {
      throw new \LogicException(sprintf('Invalid percentage: %s', $percentage));
    }
    $this->context['finished'] = $percentage / 100;
  }

  /**
   * Increment various batch context items data.
   */
  public function increment($category, $key) {
    if (!isset($this->context[$category][$key])) {
      $this->context[$category][$key] = 0;
    }
    $this->context[$category][$key]++;
  }

  /**
   * Gets the current ID of the item being processed.
   */
  public function getCurrentId() {
    return $this->context['sandbox']['current_id'];
  }

  /**
   * Increment the current ID.
   */
  public function incrementCurrentId() {
    $this->context['sandbox']['current_id']++;
  }

  /**
   * Update the progress of the batch process.
   */
  public function setProgress($progress) {
    $this->context['sandbox']['progress'] = $progress;
  }

  /**
   * Increment the progess of the batch process.
   */
  public function incrementProgress() {
    $this->increment('sandbox', 'progress');
  }

  /**
   * Gets the progress of the batch process.
   */
  public function getProgress() {
    return $this->context['sandbox']['progress'];
  }

  /**
   * Sets the total number of processed items.
   */
  public function setTotal(int $total) {
    $this->context['results']['total'] = $total;
  }

  /**
   * Gets the total number of processed items.
   */
  public function getTotal() {
    return $this->context['results']['total'];
  }

  /**
   * Increment the created attribute.
   */
  public function incrementCreated() {
    $this->increment('results', 'created');
  }

  /**
   * Increment the updated attribute.
   */
  public function incrementUpdated() {
    $this->increment('results', 'updated');
  }

  /**
   * Increment the skipped.
   */
  public function incrementSkipped() {
    $this->increment('results', 'skipped');
  }

  /**
   * Gets info on the batch process.
   */
  protected function getBatchInfo(): array {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function getPageNumber() : int {
    return (int) ceil(($this->getCurrentId() + 1) / static::ITEMS_PER_PAGE);
  }

  /**
   * {@inheritdoc}
   */
  public function processBatchItem() {
    $this->setFinished();
  }

  /**
   * {@inheritdoc}
   */
  public function batchProcessingFinished(array $results, $duration) {
    if (empty($results['created'])) {
      $this->messenger->addStatus(t('No new items created.'));
    }
    else {
      $this->messenger->addStatus(t('It took @duration to create @count items.', [
        '@count' => $results['created'],
        '@duration' => $duration,
      ]));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function batchProcessingFailed($failure) {
    // TODO: revise to make this error message more useful.
    $this->messenger->addError(t('Processing failed.'));
  }

}
